#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define GRID_RIGHT_CH PORTB4
#define GRID_LEFT_CH PORTB3

volatile bool _grid_sw = false;

unsigned long previousMillis = 0;
const long rise_interval = 5;

signed char _channel_right[14] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
signed char _channel_left[14] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

void setup() 
{
  // Инициализация выходов
  DDRD = DDRD | 0b11111111; //Устанавливаем все выводы порта D как выходы
  DDRB = DDRB | 0b11011011; 
  DDRC = DDRC | 0b00110000; //Устанавливаем выводы 5 и 4 порта C как выходы, остальные не изменяем
			  //0b76543210
  
  //Выключаем сетки обоих каналов
  PORTB |= (1 << GRID_RIGHT_CH) | (1 << GRID_LEFT_CH); //HIGH

  //Включаем опорное напряжение с ноги AREF
  analogReference(EXTERNAL);
   
  //-------------------------------------------------------------    
  // инициализация Timer1
  cli(); // отключить глобальные прерывания
  TCCR1A = 0; // установить регистры в 0
  TCCR1B = 0; 

  OCR1A = 15624; // установка регистра совпадения
  TCCR1B |= (1 << WGM12); // включение в CTC режим

  // Установка битов CS10 и CS12 на коэффициент деления
  TCCR1B |= (1<<CS10); //Деление на 1
  // TCCR1B |= (1<<CS11);           // CLK/8
  // TCCR1B |= (1<<CS10)|(1<<CS11); // CLK/64
  // TCCR1B |= (1<<CS12);           // CLK/256
  // TCCR1B |= (1<<CS10)|(1<<CS12); // CLK/1024

  TIMSK1 |= (1 << OCIE1A);  // включение прерываний по совпадению
  sei(); // включить глобальные прерывания
}

ISR(TIMER1_COMPA_vect)
{
     if(_grid_sw == false)
     {
        _grid_sw = true;

        ClearIndicator();

    	PORTB |= (1 << GRID_RIGHT_CH); //HIGH
    	PORTB &= ~(1 << GRID_LEFT_CH); //LOW

        Indicator(_channel_right);
     }
     else
     {
        _grid_sw = false;

        ClearIndicator();

    	PORTB &= ~(1 << GRID_RIGHT_CH); //LOW
    	PORTB |= (1 << GRID_LEFT_CH); //HIGH

        Indicator(_channel_left);
     }
}

void Indicator(signed char *value)
{
  if(value[0] == 1)
  {
	  PORTB &= ~(1 << PORTB1); //LOW
  }
  
  if(value[1] == 1)
  {
	  PORTB &= ~(1 << PORTB0); //LOW
  }
  
  if(value[2] == 1)
  {
	  PORTD &= ~(1 << PORTD7); //LOW
  }
  
  if(value[3] == 1)
  {
	  PORTD &= ~(1 << PORTD6); //LOW
  }
  
  if(value[4] == 1)
  {
	  PORTD &= ~(1 << PORTD5); //LOW
  }
  
  if(value[5] == 1)
  {
	  PORTB &= ~(1 << PORTB7); //LOW
  }
  
  if(value[6] == 1)
  {
	  PORTB &= ~(1 << PORTB6); //LOW
  }
  
  if(value[7] == 1)
  {
	  PORTD &= ~(1 << PORTD4); //LOW
  }
  
  if(value[8] == 1)
  {
	  PORTD &= ~(1 << PORTD3); //LOW
  }
  
  if(value[9] == 1)
  {
	  PORTD &= ~(1 << PORTD2); //LOW
  }
  
  if(value[10] == 1)
  {
	  PORTD &= ~(1 << PORTD1); //LOW
  }
  
  if(value[11] == 1)
  {
	  PORTD &= ~(1 << PORTD0); //LOW
  }
  
  if(value[12] == 1)
  {
	  PORTC &= ~(1 << PORTC5); //LOW
  }
  
  if(value[13] == 1)
  {
	  PORTC &= ~(1 << PORTC4); //LOW
  }  
}

void ClearIndicator()
{
  //Гашение всех символов
  PORTD = PORTD | 0b11111111; //HIGH
  PORTB = PORTB | 0b11011011; //HIGH
  PORTC = PORTC | 0b00110000; //HIGH
}

void SetToIndicationBuffer(signed char *channel, int value)
{
  if(value >= 25)
  {
    channel[0] = 1;

    Delay_while(rise_interval);
  }
  else
  {
    channel[0] = 0;
  }  
    
  if(value >= 61)
  {
    channel[1] = 1;

    Delay_while(rise_interval);
  }
  else
  {
    channel[1] = 0;
  }  
    
  if(value >= 86)
  {
    channel[2] = 1;

    Delay_while(rise_interval);
  }
  else
  {
    channel[2] = 0;
  }  
    
  if(value >= 115)
  {
    channel[3] = 1;

    Delay_while(rise_interval);
  }
  else
  {
    channel[3] = 0;
  }  
    
  if(value >= 135)
  {
    channel[4] = 1;

    Delay_while(rise_interval);
  }
  else
  {
    channel[4] = 0;
  }  
    
  if(value >= 151)
  {
    channel[5] = 1;

    Delay_while(rise_interval);
  }
  else
  {
    channel[5] = 0;
  }
    
  if(value >= 181)
  {
    channel[6] = 1;

    Delay_while(rise_interval);
  }
  else
  {
    channel[6] = 0;
  }
    
  if(value >= 229)
  {
    channel[7] = 1;
    
    Delay_while(rise_interval);
  }
  else
  {
    channel[7] = 0;
  }
    
  if(value >= 308)
  {
    channel[8] = 1;

    Delay_while(rise_interval);
  }
  else
  {
    channel[8] = 0;
  }
    
  if(value >= 437)
  {
    channel[9] = 1;

    Delay_while(rise_interval);
  }
  else
  {
    channel[9] = 0;
  }
    
  if(value >= 542)
  {
    channel[10] = 1;

    Delay_while(rise_interval);
  }
  else
  {
    channel[10] = 0;
  }
    
  if(value >= 607)
  {
    channel[11] = 1;

    Delay_while(rise_interval);
  }
  else
  {
    channel[11] = 0;
  }
    
  if(value >= 681)
  {
    channel[12] = 1;

    Delay_while(rise_interval);
  }
  else
  {
    channel[12] = 0;
  }
    
  if(value >= 821)
  {
    channel[13] = 1;

    Delay_while(rise_interval);
  }  
  else
  {
    channel[13] = 0;
  } 
}

void Delay_while(long interval)
{
  while(1)
  {
    unsigned long currentMillis = millis();
    if (currentMillis - previousMillis >= interval)
    {
      previousMillis = currentMillis;
      break;
    }
  }
}

void loop() 
{
    int adc_right_ch = analogRead(A0);		  
	SetToIndicationBuffer(_channel_right, adc_right_ch);
      	  
    int adc_left_ch = analogRead(A1);
    SetToIndicationBuffer(_channel_left, adc_left_ch);  
}
